<?php

namespace Litepie\Notification\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Notification\Events\NotificationAction as NotificationActionEvent;
use Litepie\Notification\Notifications\NotificationAction as NotificationActionNotification;
use Litepie\Actions\Concerns\AsAction;

class NotificationAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the NotificationActionEvent.
     *
     * @param   NotificationActionEvent  $event
     * @return mixed
     */
    public function handle(NotificationActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $notification.
     *
     * @param   NotificationActionEvent  $event
     * @return void
     */
    public function create(NotificationActionEvent $event)
    {
        $client = $event->notification->client;
        Notification::send($client, new NotificationActionNotification($event));
    }

    /**
     * Handle the NotificationActionEvent as a listener.
     *
     * @param   NotificationActionEvent  $event
     * @return mixed
     */
    public function asListener(NotificationActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   NotificationActionEvent  $event
     * @return bool
     */
    private function isAllowed(NotificationActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
