<?php

namespace Litepie\Notification\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Litepie\Notification\Models\Notification;

class NotificationPolicy
{


    /**
     * Determine if the given user can view the notification.
     *
     * @param Authenticatable $user
     * @param Notification $notification
     *
     * @return bool
     */
    public function view(Authenticatable $user, Notification $notification)
    {
        if ($authUser->canDo('notification.notification.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $notification->is_owner;
    }

    /**
     * Determine if the given user can create a notification.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('notification.notification.create');
    }

    /**
     * Determine if the given user can update the given notification.
     *
     * @param Authenticatable $user
     * @param Notification $notification
     *
     * @return bool
     */
    public function update(Authenticatable $user, Notification $notification)
    {
        if ($user->canDo('notification.notification.edit') && $user->isAdmin()) {
            return true;
        }

        return $notification->is_owner;
    }

    /**
     * Determine if the given user can delete the given notification.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Notification $notification)
    {
        return $notification->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
